/**
 *
 *  This file is part of iliad-regutils.
 *
 *  Copyright (C) 2008 Martijn van der Horst <M.G.v.d.Horst@gmail.com>
 * 
 *  iliad-regutils is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <argp.h>
#include <libgen.h>
#include "regutil.h"
#include "regutil_utilities.h"
#include "vstrcat.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
#include <libxml/xmlstring.h>
#include <libxml/globals.h>

/* Command line parsing */
error_t global_parser_callback(int key, char *arg, struct argp_state *state) {
  error_t result = 0;
  global_arguments *arguments = state->input;

  switch (key) {
    case 'v':
      arguments->verbose = 1;
      break;
    case 'f':
      arguments->file = arg;
      break;
    default:
      result = arguments->utility_callback(key, arg, state);
  }
  return result;
}

// Version and name of the program
const char *argp_program_version = "regutil 0.1";

// Bug address
const char *argp_program_bug_address = "<M.G.v.d.Horst@gmail.com>";

int main(int argc, char **argv) {
  int error = 0;

  // Identity the utility by examining the name used to call this executable
  int util_id = identify_utility(basename(argv[0]));

  // Get a parser structure and arguments
  struct argp *parser = get_parser(util_id);
  global_arguments *arguments = get_argument_structure(util_id);
  
  // Parse the command line options
  argp_parse(parser, argc, argv, 0, 0 , arguments);

  // Initialize libxml
  xmlInitParser();
  xmlKeepBlanksDefault(0);

  // Load registry
  xmlDocPtr registry = xmlParseFile(arguments->file);
  if (registry == NULL) {
    fprintf(stderr, "Error: unable to parse file \"%s\"\n", arguments->file);
    error = -1;
  }
  
  if (!error) {

    // Call the utility
    utility_main main_func = get_utility_main(util_id);
    error = main_func(registry, arguments);
 
    // Save the registry, if it has been changed by the utility
    if ((!error) && (arguments->changed)) {
      error = xmlSaveFormatFile(arguments->file, registry, 1);
      if (error == -1) fprintf(stderr, "Error: unable to save file \"%s\"\n", arguments->file);
      else error = 0;
    }

    // Free the registry    
    xmlFreeDoc(registry);
  }

  // Shutdown libxml
  xmlCleanupParser();

  // Cleanup the parser structure and arguments
  free(parser);
  free_argument_structure(arguments);
    
  return error;
}

/* This function evaluates the xpath expression and returns the node set associated with the expression.
 * The caller is responsible for freeing the nodeset with a call to xmlXPathFreeNodeSet.
 * This function returns NULL when an error occurs;
 */
xmlNodeSetPtr evaluate_xpath(xmlDocPtr registry, xmlChar *xpath, int verbose) {
  int error = 0;
  xmlNodeSetPtr result = NULL;
  xmlXPathContextPtr xpathCtx; 

  // Create xpath context
  xpathCtx = xmlXPathNewContext(registry);
  if(xpathCtx == NULL) {
    fprintf(stderr,"Error: unable to create new XPath context\n");
    error = -1;
  }

  if (!error) {
    xmlXPathObjectPtr xpathObj;
    
    if (verbose) fprintf(stdout,"Evaluating \"%s\"\n", xpath);
    
    /* Evaluate xpath expression */
    xpathObj = xmlXPathEvalExpression(xpath, xpathCtx);
    if(xpathObj == NULL) {
      fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", xpath);
      error = -1;
    } else result = xpathObj->nodesetval;
    
    /* Cleanup of XPath data */
    xmlXPathFreeNodeSetList(xpathObj);
    xmlXPathFreeContext(xpathCtx); 
  }
  
  return result;
}

/* This function removes the nodes from the registry
 * it returns the number of nodes removed
 */
int remove_nodes(xmlNodeSetPtr nodes, int verbose) {
    int i;

    if (verbose) fprintf(stdout, "Removing %d nodes\n", nodes->nodeNr);
    
    /* We need to do this in reversed order because some of the nodes may be sub-nodes of earlier nodes */
    for(i = nodes->nodeNr - 1; i >= 0; i--) {
        xmlUnlinkNode(nodes->nodeTab[i]);
        xmlFreeNode(nodes->nodeTab[i]);
        nodes->nodeTab[i]= NULL;
    }
    
    return nodes->nodeNr;
}

/* This function updates the value of the nodes in the registry
 * it returns the number of nodes modified
 */
int update_nodes(xmlNodeSetPtr nodes, xmlChar *newval, int verbose) {
    int i;

    if (verbose) fprintf(stdout, "Setting %d nodes to the value \"%s\"\n", nodes->nodeNr, newval);
    
    /* We need to do this in reversed order because some of the nodes may be sub-nodes of earlier nodes */
    for(i = nodes->nodeNr - 1; i >= 0; i--) {
        xmlNodeSetContent(nodes->nodeTab[i],newval);
    }
    
    return nodes->nodeNr;
}

/* This function prints the value of nodes to stdout */
void show_nodes(xmlNodeSetPtr nodes, int verbose) {
  if (nodes != NULL) {
    int i;
    for (i = 0; i < nodes->nodeNr; i++) {
      xmlNodePtr node = nodes->nodeTab[i];
      fprintf(stdout,"%s\n", xmlNodeGetContent(node));
    }
  }
}

/* This function is used to access child nodes of a parent node
 * The xpath_parent XPath should lead to exactly one node when it is evaluated on registry
 * The child parameter indicates the name of the child field
 * The newval parameter contains the new value of the child, it may also contain NULL in which case the child is not updated
 * The print_old parameter indicates whether the old value of the child, before updating, should be printed to stdout
 * The return value of the function is larger than 0 if the child was updated and smaller than 0 if an error occured
 */
int access_node(xmlDocPtr registry, xmlChar *xpath_parent, xmlChar *child, xmlChar *newval, int print_old, int verbose) {
  int result = 0;
  char *xpath_child = vstrcat(xpath_parent,"/", child, NULL);
  xmlNodeSetPtr nodes_child = evaluate_xpath(registry, xpath_child, verbose);

  if (nodes_child != NULL) {      
    // Show the elements
    if (print_old) show_nodes(nodes_child, verbose);
      
    // Set the elements
    if (newval != NULL) {
      // Determine whether the child node exists
      if (nodes_child->nodeNr != 0) {
        // The node exists, update it:
	result = update_nodes(nodes_child, newval, verbose);
      } else {
        // The node does not exists, we need to add it to the parent
	xmlNodeSetPtr nodes_parent = evaluate_xpath(registry, xpath_parent, verbose);

	if (nodes_parent != NULL) {
	  // Add the child to the parent node
	  xmlNewChild(nodes_parent->nodeTab[0], NULL, child, newval);
	  result = 1;
	} else result = -1;

        xmlXPathFreeNodeSet(nodes_parent);
      }
    }
  } else result = -1;

  // Cleanup
  xmlXPathFreeNodeSet(nodes_child);
  free(xpath_child);
  
  return result;
}
