/**
 *
 *  This file is part of iliad-regutils.
 *
 *  Copyright (C) 2008 Martijn van der Horst <M.G.v.d.Horst@gmail.com>
 * 
 *  iliad-regutils is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef REGUTIL_H
#define REGUTIL_H

#include <argp.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

#define ILIAD_REGISTRY "/mnt/settings/registry/registry.xml"

/* The iliad-regutils package consists of multiple utilities
 * bundled in one executable.
 * The name of the executable determines the utility that is executed.
 * The following data types are used to describe each utility
 */

// Global callback function
// This function parses the global functions and then calls the parser function of the utility
error_t global_parser_callback(int key, char *arg, struct argp_state *state);

// A callback function for the parsing the command lines
typedef error_t (*parser_callback)(int key, char *arg, struct argp_state *state);

// Global options
// All utilities must have these options in their utility_options array
#define GLOBAL_OPTIONS \
  {"verbose", 'v', 0,      0, "Produce verbose output"}, \
  {"file",    'f', "FILE", 0, "Read/modify FILE instead of the registry"},

// Global arguments
// This structure describes the global arguments passed to the utility,
// it contains a pointer to the utility's callback function,
// it contains a pointer to the argument type of the specific utility,
// and it provides the utility a way to indicate whether it updated the registry
typedef struct {
  int verbose;                      // Boolean indicating whether to produce verbose output
  char *file;                       // The XML file to modify, by default this will be the registry
  parser_callback utility_callback; // Callback function of the utility
  void *utility_arguments;          // Pointer to the arguments structure of the utility
  int changed;                      // Boolean indicating whether the utility changed the registry
} global_arguments;

// The main function of a utility
typedef int (*utility_main)(xmlDocPtr registry, global_arguments *arguments);

// Each utility describes itself using the following struct:
typedef struct {
  char *name;                           // Name of the utility
  struct argp_option *options;          // Command line options of the utility
  char *args_doc;                       // Non-option arguments of the utility
  char *doc;                            // Documentation of the utility
  size_t argument_size;                 // Size of the argument structure of the utility
  void (*init_arguments)(void *args);    // Function that initializes the argument structure
  parser_callback callback;             // Callback function for command line parsing
  utility_main main;                    // Main function of the utility
} utility_description;

/* The rest of this file contains functions that can be used by the utilities in examining or modifying the registry */

// A function for finding the nodes in the registry that are associated with a certain xpath
xmlNodeSetPtr evaluate_xpath(xmlDocPtr registry, xmlChar *xpath, int verbose);

// A function for removing nodes from the registry
int remove_nodes(xmlNodeSetPtr nodes, int verbose);

// A function for setting the value of nodes
int update_nodes(xmlNodeSetPtr nodes, xmlChar *newval, int verbose);

// A function for the printing of the value of nodes to stdout
void show_nodes(xmlNodeSetPtr nodes, int verbose);

// A function for accessing the child node of a parent node
int access_node(xmlDocPtr registry, xmlChar *xpath_parent, xmlChar *child, xmlChar *newval, int print_old, int verbose);

#endif /* REGUTIL_H */
