/**
 *
 *  This file is part of iliad-regutils.
 *
 *  Copyright (C) 2008 Martijn van der Horst <M.G.v.d.Horst@gmail.com>
 * 
 *  iliad-regutils is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "regutil.h"
#include "regutil_application_install.h"
#include "regutil_extension.h"
#include "regutil_utilities.h"
#include <libxml/parser.h>
#include "vstrcat.h"

#define APPLICATION_INSTALL_MAX_EXTENSIONS 100

void regutil_application_install_init(void *args) {
  regutil_application_install_arguments *arguments = (regutil_application_install_arguments *)args;
  int i;

  // Set the default values in the argument structure
  arguments->application = NULL;
  arguments->appCategory = NULL;
  arguments->argMask = NULL;
  arguments->execute = NULL;
  arguments->xResourceName = NULL;
  arguments->ipcChannel = NULL;
  arguments->appIconPath = NULL;
  arguments->exticon = NULL;
  arguments->extensions = calloc(APPLICATION_INSTALL_MAX_EXTENSIONS+1, sizeof(char *));
  for (i = 0; i < (APPLICATION_INSTALL_MAX_EXTENSIONS+1); i++) arguments->extensions[i] = NULL;
}

error_t regutil_application_install_callback(int key, char *arg, struct argp_state *state) {
  error_t result = 0;
  global_arguments *arguments = state->input;
  regutil_application_install_arguments *util_arguments = arguments->utility_arguments;

  switch (key) {
    case 'c':
      util_arguments->appCategory = arg;
      break;
    case 'm':
      util_arguments->argMask = arg;
      break;
    case 'e':
      util_arguments->execute = arg;
      break;
    case 'x':
      util_arguments->xResourceName = arg;
      break;
    case 'p':
      util_arguments->ipcChannel = arg;
      break;
    case 'i':
      util_arguments->appIconPath = arg;
      break;
    case 'I':
      util_arguments->exticon = arg;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num == 0) util_arguments->application = arg; // This is the APPLICATION
      else if (((state->arg_num)-1) < APPLICATION_INSTALL_MAX_EXTENSIONS) util_arguments->extensions[state->arg_num-1] = arg; // This is an EXTENSION
      else argp_usage(state);                                     // This is one argument too many
      break;     
    case ARGP_KEY_END:
      if (state->arg_num == 0) argp_usage(state); // The APPLICATION argument was not given
      break;
    default:
      result = ARGP_ERR_UNKNOWN;
  }

  return result;
}

int regutil_application_install_main(xmlDocPtr registry, global_arguments *arguments) {
  int error = 0;
  regutil_application_install_arguments *util_arguments = arguments->utility_arguments;
  char *xpath = vstrcat("/iLiadRegistry/applications/application[@type='",util_arguments->application,"']",NULL);

  // Evaluate the xpath expression  
  xmlNodeSetPtr nodes = evaluate_xpath(registry, xpath, arguments->verbose);

  if (nodes == NULL) {
    fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", xpath);
    error = -1;
  } else {
  
    // Determine whether the application is already in the registry or not
    if (nodes->nodeNr == 0) {
      // The application does not exist
      if (arguments->verbose) fprintf(stdout,"The application was not found in the registry\n");

      // We need to add the node
      char *applications_xpath = "/iLiadRegistry/applications";
      xmlNodeSetPtr applications = evaluate_xpath(registry, applications_xpath, arguments->verbose);
	
      // Sanity check
      if ((applications == NULL) || (applications->nodeNr != 1)) {
	if (applications == NULL) fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", xpath);
        else fprintf(stderr,"Error: Your registry seems corrupted (%d applications sections found)\n", applications->nodeNr);
	error = -1;
      } else {
	// Add the application and its settings
	
	int i, id;
	xmlNodePtr support;
	global_arguments *globargs;
	regutil_extension_arguments *extargs;
	utility_main regutil_extension;
	
	// Add the application node
	xmlNodePtr application = xmlNewChild(applications->nodeTab[0], NULL, "application", NULL);
	xmlNewProp(application, "type", util_arguments->application);
	
	// Add the children, if any
	xmlNewChild(application, NULL, "appCategory", util_arguments->appCategory);
	xmlNewChild(application, NULL, "argMask", util_arguments->argMask);
	xmlNewChild(application, NULL, "execute", util_arguments->execute);
	support = xmlNewChild(application, NULL, "supportFormat", NULL);
	for (i = 0; util_arguments->extensions[i] != NULL; i++) {
	  xmlNewChild(support, NULL, "extName", util_arguments->extensions[i]);
	}
	xmlNewChild(application, NULL, "xResourceName", util_arguments->xResourceName);
	xmlNewChild(application, NULL, "ipcChannel", util_arguments->ipcChannel);
	xmlNewChild(application, NULL, "appIconPath", util_arguments->appIconPath);

	// Add extensions by calling the registry-extension utility
	id = identify_utility("registry-extension");
	globargs = get_argument_structure(id);
        regutil_extension = get_utility_main(id);
	// Copy the necessary arguments: (FIXME: Make a nice clone function for this)
	globargs->verbose = arguments->verbose;
	// Set the arguments of the utility
	extargs = globargs->utility_arguments;
	extargs->app = util_arguments->application; // Set the application
	extargs->icon = util_arguments->exticon; // Set the icon
	for (i = 0; util_arguments->extensions[i] != NULL; i++) {
	  extargs->extension = util_arguments->extensions[i];
	  // FIXME: Add error handling
	  regutil_extension(registry, globargs);
	}
	free_argument_structure(globargs);
	
	// Mark the registry as changed
	arguments->changed = 1;
	
      }
	
      xmlXPathFreeNodeSet(applications);

    } else if (nodes->nodeNr == 1) {
      // The application exists
      // Do nothing
      fprintf(stderr,"The application \"%s\" is already installed in the registry\n", util_arguments->application);      
      error = -1;
    } else {
      // There are multiple entries
      // PANIC
      // FIXME: Do something other than panic
      fprintf(stderr,"The application \"%s\" is in the registry %d times.\nRefusing to continue\n", util_arguments->application,
      nodes->nodeNr);
      error = -1;
    }
  
    // Cleanup
    xmlXPathFreeNodeSet(nodes);
  }

  free(xpath);

  free(util_arguments->extensions);
  
  return error;
}
