/**
 *
 *  This file is part of iliad-regutils.
 *
 *  Copyright (C) 2008 Martijn van der Horst <M.G.v.d.Horst@gmail.com>
 * 
 *  iliad-regutils is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "regutil.h"
#include "regutil_extension.h"
#include <libxml/parser.h>
#include "vstrcat.h"

void regutil_extension_init(void *args) {
  regutil_extension_arguments *arguments = (regutil_extension_arguments *)args;

  // Set the default values in the argument structure
  arguments->extension = NULL;
  arguments->delete = 0;
  arguments->show_icon = 0;
  arguments->icon = NULL;
  arguments->show_app = 0;
  arguments->app = NULL;
}

error_t regutil_extension_callback(int key, char *arg, struct argp_state *state) {
  error_t result = 0;
  global_arguments *arguments = state->input;
  regutil_extension_arguments *util_arguments = arguments->utility_arguments;

  switch (key) {
    case 'd':
      util_arguments->delete = 1;
      break;
    case 'i':
      util_arguments->show_icon = 1;
      break;
    case 'I':
      util_arguments->icon = arg;
      break;
    case 'a':
      util_arguments->show_app = 1;
      break;
    case 'A':
      util_arguments->app = arg;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num == 0) util_arguments->extension = arg; // This is the EXTENSION
      else argp_usage(state);                                   // This is one argument too many
      break;     
    case ARGP_KEY_END:
      if (state->arg_num != 1) argp_usage(state); // The EXTENSION argument was not given
      break;
    default:
      result = ARGP_ERR_UNKNOWN;
  }

  return result;
}

int regutil_extension_main(xmlDocPtr registry, global_arguments *arguments) {
  int error = 0;
  regutil_extension_arguments *util_arguments = arguments->utility_arguments;
  char *xpath = vstrcat("/iLiadRegistry/extensionInfos/extensionInfo[extName='",util_arguments->extension,"']",NULL);

  // Evaluate the xpath expression  
  xmlNodeSetPtr nodes = evaluate_xpath(registry, xpath, arguments->verbose);

  if (nodes == NULL) {
    fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", xpath);
    error = -1;
  } else {
  
    // Determine whether the extension is already in the registry or not
    if (nodes->nodeNr == 0) {
      // The extension does not exist
      if (arguments->verbose) fprintf(stdout,"Extension was not found in the registry\n");

      // So we don't print anything, and we can skip the deletion
      if ((!util_arguments->delete) && ((util_arguments->icon != NULL) || (util_arguments->app !=NULL))) {
        // We need to add the node
	char *extensionInfos_xpath = "/iLiadRegistry/extensionInfos";
	xmlNodeSetPtr extensionInfos = evaluate_xpath(registry, extensionInfos_xpath, arguments->verbose);
	
	// Sanity check
	if ((extensionInfos == NULL) || (extensionInfos->nodeNr != 1)) {
	  if (extensionInfos == NULL) fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", xpath);
          else fprintf(stderr,"Error: Your registry seems corrupted (%d extensionInfos sections found)\n", extensionInfos->nodeNr);
	  error = -1;
	} else {
	  // Continue to add the extension and its settings
	  xmlNodePtr extensionInfo = xmlNewChild(extensionInfos->nodeTab[0], NULL, "extensionInfo", NULL);
	  // Add the application, if any
	  if (util_arguments->app != NULL) xmlNewChild(extensionInfo, NULL, "associateApp", util_arguments->app);
	  // Add the extension name
	  xmlNewChild(extensionInfo, NULL, "extName", util_arguments->extension);
	  // Add the icon, if any
	  if (util_arguments->icon != NULL) xmlNewChild(extensionInfo, NULL, "extIcon", util_arguments->icon);
	  // Mark the registry as changed
	  arguments->changed = 1;
	}
	
	xmlXPathFreeNodeSet(extensionInfos);
      }

    } else if (nodes->nodeNr == 1) {
      // The extension exists
      
      // Access the elements
      // FIXME: We're ignoring any possible errors here
      if (access_node(registry, xpath, "associateApp", util_arguments->app, util_arguments->show_app, arguments->verbose) > 0) arguments->changed = 1;
      if (access_node(registry, xpath, "extIcon", util_arguments->icon, util_arguments->show_icon, arguments->verbose) > 0) arguments->changed = 1;

      // Finally delete the node, if necessary
      if (util_arguments->delete) arguments->changed = remove_nodes(nodes, arguments->verbose);

    } else {
      // There are multiple entries
      // PANIC
      // FIXME: Do something other than panic
      fprintf(stderr,"The extension \"%s\" is in the registry %d times.\nRefusing to continue\n", util_arguments->extension,
      nodes->nodeNr);
      error = -1;
    }
  
    // Cleanup
    xmlXPathFreeNodeSet(nodes);
  }

  free(xpath);
  
  return error;
}
